--[[
	BActionBar - A Bongos Actionbar
--]]

BActionBar = CreateFrame('Button')
setmetatable(BActionBar, {__index = BBar})
BActionBar.super = BBar

local Bar_mt = {__index = BActionBar}

local DEFAULT_SPACING = 2
local DEFAULT_SIZE = 12
local DEFAULT_COLS = 12
local MAX_BUTTONS = BActionButton.GetMax()
local BUTTON_SIZE = BActionButton.GetSize()

local function AddState(stateStr, state)
	if stateStr then
		return stateStr .. ',' .. state
	end
	return state
end


--[[ Rightclick Menu Functions ]]--

local function StateButton_OnEnter()
	GameTooltip:SetOwner(this, 'ANCHOR_TOPRIGHT')
	GameTooltip:SetText(this:GetText(), 1, 1, 1)

	if this:GetChecked() then
		GameTooltip:AddLine(BONGOS_STATE_HIDE)
		GameTooltip:AddLine(BONGOS_STATE_INVERSE_SELECT)
	else
		GameTooltip:AddLine(BONGOS_STATE_SHOW)
		GameTooltip:AddLine(BONGOS_STATE_SELECT_ONLY)
	end
	GameTooltip:Show()
end

local function StateButton_OnLeave()
	GameTooltip:Hide()
end

local function StateButton_OnClick()
	local menu = this:GetParent()
	local name = menu:GetName()
	local enable = this:GetChecked()

	if IsControlKeyDown() then
		local s,e = BState.GetStanceRange()
		for i = s, e do
			local button = getglobal(name .. 'Stance' .. i)
			if button ~= this then
				menu.frame:AddState('s' .. i, not enable)
				button:SetChecked(not enable)
			end
		end

		local maxPage = BState.GetMaxPage()
		for i = 2, maxPage do
			local button = getglobal(name .. 'Page' .. i)
			if button ~= this then
				menu.frame:AddState('p' .. i-1, not enable)
				button:SetChecked(not enable)
			end
		end
	end
	menu.frame:AddState(this.id, enable)
	StateButton_OnEnter()
end

local function StateButton_Create(name, id, parent)
	local button = CreateFrame('CheckButton', name, parent, 'GooeyCheckButton')
	button.id = id
	button:SetText(BState.GetStateName(id))
	button:SetScript('OnClick', StateButton_OnClick)
	button:SetScript('OnEnter', StateButton_OnEnter)
	button:SetScript('OnLeave', StateButton_OnLeave)
	
	return button
end

local function AddStanceButtons(menu, name)
	--stances
	local s, e = BState.GetStanceRange()
	for i = s, e do
		local button = StateButton_Create(name .. 'Stance' .. i, 's' .. i, menu)
		if i == s then
			button:SetPoint('TOPLEFT', menu, 'TOPLEFT', 6, -28)
		else
			button:SetPoint('TOP', name .. 'Stance' .. (i-1), 'BOTTOM', 0, 2)
		end
	end

	--pages
	local maxPage = BState.GetMaxPage()
	for i = 2, maxPage do
		local button = StateButton_Create(name .. 'Page' .. i, 'p' .. i-1, menu)
		if i > 2 then
			button:SetPoint('TOP', name .. 'Page' .. (i-1), 'BOTTOM', 0, 2)
		else
			button:SetPoint('TOP', name .. 'Stance' .. e, 'BOTTOM', 0, 2)
		end
	end

	menu:SetWidth(220)
	menu:SetHeight(260 + (e - s + maxPage) * 30)
end

--[[
local function CreateStanceMenu(name, parent)
	local menu = CreateFrame('Frame', name .. 'Visibility', parent)
	menu:SetAllPoints(parent)
	
	local s,e = BState.GetStanceRange()
	for i = s, e do
	end
end
--]]

local function CreateConfigMenu(name)
	local menu = CreateFrame("Button", name, UIParent, "BongosRightClickMenu")

	--checkbuttons
	AddStanceButtons(menu, name)

	--sliders
	local opacity = CreateFrame("Slider", name .. "Opacity", menu, "BongosOpacitySlider")
	opacity:SetPoint("BOTTOM", menu, "BOTTOM", 0, 24)

	local scale = CreateFrame("Slider", name .. "Scale", menu, "BongosScaleSlider")
	scale:SetPoint("BOTTOM", opacity, "TOP", 0, 24)

	local spacing = CreateFrame("Slider", name .. "Spacing", menu, "BongosSpaceSlider")
	spacing:SetPoint("BOTTOM", scale, "TOP", 0, 24)
	spacing:SetScript("OnValueChanged", function()
		if not menu.onShow then
			menu.frame:SetSpacing(this:GetValue())
		end
		getglobal(this:GetName() .. 'ValText'):SetText(this:GetValue())
	end)

	local cols = CreateFrame("Slider", name .. "Cols", menu, "BongosSlider")
	cols:SetPoint("BOTTOM", spacing, "TOP", 0, 24)
	cols:SetScript("OnValueChanged", function()
		if not menu.onShow then
			menu.frame:SetColumns(menu.frame:GetSize() - this:GetValue() + 1)
		end
		getglobal(this:GetName() .. 'ValText'):SetText(menu.frame:GetColumns())
	end)
	cols:SetValueStep(1)
	getglobal(name .. "ColsText"):SetText(BONGOS_COLUMNS)
	getglobal(name .. "ColsHigh"):SetText(1)

	local size = CreateFrame("Slider", name .. "Size", menu, "BongosSlider")
	size:SetPoint("BOTTOM", cols, "TOP", 0, 24)
	size:SetScript("OnValueChanged", function()
		if not menu.onShow then
			menu.frame:SetSize(this:GetValue())
		end
		getglobal(this:GetName() .. 'ValText'):SetText(this:GetValue())

		local size = menu.frame:GetSize()
		local cols = menu.frame:GetColumns()
		getglobal(name .. 'Cols'):SetMinMaxValues(1, size)
		getglobal(name .. 'Cols'):SetValue(size - cols + 1)
		getglobal(name .. "ColsLow"):SetText(size)
		getglobal(name .. "ColsValText"):SetText(cols)

	end)
	size:SetValueStep(1)
	getglobal(name .. "SizeText"):SetText(BONGOS_SIZE)
	getglobal(name .. "SizeLow"):SetText(1)
	
	return menu
end


--[[ Constructor ]]--

local oDragFrame_OnMouseUp
local function DragFrame_OnMouseUp()
	oDragFrame_OnMouseUp()

	if arg1 == 'LeftButton' then
		local self = this.parent
		for i = 1, BActionBar.GetNumber() do
			if i ~= self.id then
				local bar = BActionBar.Get(i)
				if MouseIsOver(bar) and not(bar:GetDock() or bar:IsDependent(self)) then
					self:Dock(bar)
					break
				end
			end
		end
	end
end

local oDragFrame_OnEnter
local function DragFrame_OnEnter()
	oDragFrame_OnEnter()
	GameTooltip:AddLine(BONGOS_DOCK_HELP)

	local bar = this.parent
	if bar.sets.vis then
		local used
		local s,e = BState.GetStanceRange()
		for i = s, e do
			local stance = 's' .. i
			if bar:HasState(stance) then
				used = true
				GameTooltip:AddLine(format(BONGOS_STATE_SHOWN_IN, BState.GetStateName(stance)), 0, 1, 0)
			end
		end

		local maxPage = BState.GetMaxPage()
		for i = 2, maxPage do
			local page = 'p' .. i-1
			if bar:HasState(page) then
				used = true
				GameTooltip:AddLine(format(BONGOS_STATE_SHOWN_ON_PAGE, i), 0, 1, 0)
			end
		end	
		
		if not used then
			GameTooltip:AddLine(BONGOS_STATE_UNUSED, 1, 0.2, 0.2)
		end
	end
	GameTooltip:Show()
end

local function OnDelete(self)
	self:RefreshBindings()
	
	for i = self:GetStartID(), self:GetEndID() do
		local button = BActionButton.Get(i)
		if button then
			button:Release()
		end
	end
	self.showstates = nil
end

local function OnCreate(self)
	--add tooltips to the dragFrame
	if not oDragFrame_OnMouseUp then
		oDragFrame_OnMouseUp = self.dragFrame:GetScript('OnMouseUp')
	end
	if not oDragFrame_OnEnter then
		oDragFrame_OnEnter = self.dragFrame:GetScript('OnEnter')
	end

	self.dragFrame:SetScript('OnMouseUp', DragFrame_OnMouseUp)
	self.dragFrame:SetScript('OnEnter', DragFrame_OnEnter)
end


function BActionBar.Create(id)
	if not Bongos.GetBarSets(id) then
		Bongos.SetBarSets(id, {vis = 1, states = ALL_STATES})
	end

	local bar = BBar.Create(id, OnCreate, OnDelete, 'SecureStateDriverTemplate')
	setmetatable(bar, Bar_mt)

	if not bar:IsUserPlaced() then
		local start = bar:GetStartID()
		local row = mod(start-1, 12)
		local col = ceil(start / 12) - 1
		bar:SetPoint('CENTER', UIParent, 'CENTER', 36 * row, -36 * col)
	end	

	if bar:GetDock() then
		bar.dragFrame:Hide()
	end
	
	bar:Layout()
	bar:UpdateTabs()

	BState.LoadMaps(bar)
	for i = 2, 5 do
		bar:SetUnit(i, BActionConfig.GetUnit(i))
	end
	bar:RefreshBindings()

	return bar
end

--Called when the right click menu is shown, loads the correct values to the checkbuttons/sliders/text
function BActionBar:ShowMenu()
	local name = 'BongosActionBarMenu'
	local menu = getglobal(name) or CreateConfigMenu(name)

	menu:SetText(format('Action Bar %s', self.id))
	menu.frame = self
	menu.onShow = 1

	local s, e = BState.GetStanceRange()
	for i = s, e do
		getglobal(name .. 'Stance' .. i):SetChecked(self:HasState('s' .. i))
	end

	local maxPage = BState.GetMaxPage()
	for i = 2, maxPage do
		getglobal(name .. 'Page' .. i):SetChecked(self:HasState('p' .. i-1))
	end

	getglobal(name .. 'Size'):SetMinMaxValues(1, self:GetMaxSize())
	getglobal(name .. 'Size'):SetValue(self:GetSize())
	getglobal(name .. 'SizeHigh'):SetText(self:GetMaxSize())
	getglobal(name .. 'Cols'):SetValue(self:GetSize() - self:GetColumns() + 1)
	getglobal(name .. 'Spacing'):SetValue(self:GetSpacing())
	getglobal(name .. 'Scale'):SetValue(self:GetScale() * 100)
	getglobal(name .. 'Opacity'):SetValue(self:GetAlpha() * 100)

	self:DisplayMenu(menu)

	menu.onShow = nil
end


--[[ States ]]--

function BActionBar:AddState(id, enable)
	local states = self.sets.states

	if enable then
		if not states then
			self.sets.states = id .. ';'
		elseif not states:match(id .. ';') then
			self.sets.states = states .. id .. ';'
		end
	elseif states then
		states = states:gsub(id .. ';', '')
		if states == '' then
			self.sets.states = nil
		else
			self.sets.states = states
		end
	end
	self:Refresh(true)
end

function BActionBar:HasState(id)
	local states = self.sets.states
	if states then
		return states:match(id .. ';')
	end
end

function BActionBar:Refresh(update, showAll)
	if update then self.showstates = nil end

	local starti = self:GetStartID()
	local endi = self:GetEndID()
	local showEmpty = showAll or BActionButton.ShowingEmpty()

	for i = starti, endi do
		BActionButton.Get(i):UpdateVisibility(showEmpty)
	end
	SecureStateHeader_Refresh(self)
end

function BActionBar:GetShowStates()
	if not self.showstates then
		local states = self.sets.states
		if states then
			local showstates
			for i = 1, select('#', string.split(';', states)) do
				local map = select(i, string.split(';', states))
				if map ~= '' then
					showstates = AddState(showstates, BState.MapToStates(map))
				end
			end

			if not showstates then showstates = '!*' end
			self:SetAttribute('statebindings', showstates .. ':shown;')
			
			self.showstates = showstates
		else
			self.showstates = '!*'
		end
	end
	return self.showstates
end


--[[ Layout Functions ]]--

--sizing
function BActionBar:SetSize(size)
	if not size or size == DEFAULT_SIZE then
		size = nil
	end
	self.sets.size = size
	self:Layout()
end

function BActionBar:GetSize()
	local parent = select(2, self:GetDock())
	if parent then
		return parent:GetSize()
	end
	return math.min(self.sets.size or DEFAULT_SIZE, self:GetMaxSize())
end

function BActionBar:GetMaxSize()
	return MAX_BUTTONS / BActionBar.GetNumber()
end


--columns
function BActionBar:SetColumns(cols)
	if not cols or cols == DEFAULT_COLS then
		cols = nil
	end
	self.sets.cols = cols
	self:Layout()
end

function BActionBar:GetColumns()
	local parent = select(2, self:GetDock())
	if parent then
		return parent:GetColumns()
	end
	return math.min(self.sets.cols or DEFAULT_COLS, self:GetSize())
end


--spacing
function BActionBar:SetSpacing(spacing)
	if not spacing or spacing == DEFAULT_SPACING then
		spacing = nil
	end
	self.sets.spacing = spacing
	self:Layout()
end

function BActionBar:GetSpacing()
	local parent = select(2, self:GetDock())
	if parent then
		return parent:GetSpacing()
	end
	return self.sets.spacing or DEFAULT_SPACING
end


--ids
function BActionBar:GetStartID()
	local prev = BActionBar.Get(self.id - 1)
	if prev then
		self.start = prev:GetMaxID() + 1
	else
		self.start = 1
	end
	return self.start
end

function BActionBar:GetEndID()
	return self:GetStartID() + self:GetSize() - 1
end

function BActionBar:GetMaxID()
	return self:GetStartID() + self:GetMaxSize() - 1
end


--[[ Layout ]]--

function BActionBar:Layout()
	--dont layout if this frame is docked and its parent has not been created yet
	local dock, parent = self:GetDock()
	if dock and not parent then return end

	local startID = self:GetStartID()
	local endID = self:GetEndID()
	local maxID = self:GetMaxID()

	local size = self:GetSize()
	local cols = self:GetColumns()
	local spacing = self:GetSpacing()
	local buttonSize = BUTTON_SIZE + spacing

	--size the bar
	if parent then
		self:ClearAllPoints()
		self:SetPoint('TOPLEFT', parent)
	end
	self:SetWidth(buttonSize * cols - spacing)
	self:SetHeight(buttonSize * ceil(size / cols) - spacing)

	--place all used buttons, and update those buttons showstates and hotkeys
	for i = 1, size do
		local row = mod(i-1, cols)
		local col = ceil(i / cols) - 1
		local button = BActionButton.Set(startID + i-1, self)
		button:SetPoint('TOPLEFT', self, 'TOPLEFT', buttonSize * row, -buttonSize * col)
	end
	self:Refresh()

	--remove any unused buttons
	if startID < maxID then
		for i = endID + 1, maxID do
			local button = BActionButton.Get(i)
			if button then
				button:Release()
			else break end
		end
	end

	--layout any bars docked to this one
	if not parent then
		self:ForAllDocked('Layout')
	end
end


--[[ Keybindings ]]--

function BActionBar:SetKeys(id, keys, update)
	local parent = select(2, self:GetDock())
	local relID = id - self:GetStartID() + 1

	if parent then
		parent:SetKeys(parent:GetStartID() + relID - 1, keys, update)
	else
		if self.sets[relID] ~= keys then
			self.sets[relID] = keys
			local button = BActionButton.Get(id)
			if button then
				button:SetAttribute('bindings-shown', keys)
				button:UpdateHotkey()
				if update then
					self:RefreshBindings()
				end
			end
		end

		for i = 1, BActionBar.GetNumber() do
			local bar = BActionBar.Get(i)
			if bar and bar ~= self and bar:GetDock() == self.id then
				local button = BActionButton.Get(bar:GetStartID() + relID - 1)
				if button then
					button:SetAttribute('bindings-shown', keys)
					button:UpdateHotkey()
					if update then
						bar:RefreshBindings()
					end
				end
			end
		end
	end
end

function BActionBar:GetKeys(id)
	local relID = id - self:GetStartID() + 1
	local dock = self:GetDock()
	if dock then
		return Bongos.GetBarSets(dock)[relID]
	end
	return self.sets[relID]
end

function BActionBar:CopyAllBindings(fromBar)
	local start = self:GetStartID()
	for i = 1, fromBar:GetSize() do
		self.sets[i] = nil

		local keys = fromBar.sets[i]
		local button = BActionButton.Get(start + i - 1)
		if button then
			button:SetAttribute('bindings-shown', keys)
			button:UpdateHotkey()
		end
	end
	self:RefreshBindings()
end

function BActionBar:ClearAllBindings()
	for i = self:GetStartID(), self:GetEndID() do
		self:SetKeys(i, nil) 
	end
	self:RefreshBindings()
end

function BActionBar:RefreshBindings()
	self:SetAttribute("_bindingset", nil)
	self:SetAttribute('state', 'pants')
	self:SetAttribute('state', BState.GetCurrentState())
end


--[[ Docking ]]--

function BActionBar:Dock(parent)
	local id = self.id
	
	--redock and reanchor any bars anchored to this one
	for i = 1, BActionBar.GetNumber() do
		if bar ~= self then
			local bar = BActionBar.Get(i)
			if bar:GetDock() == self.id then
				bar:Dock(parent)
			end

			local anchor, point = bar:GetPoint()
			if anchor == self then
				bar.sets.anchor = parent.id .. point
				bar:Reanchor()
			end
		end
	end
	
	--slacker way to load settings
	self:Delete()

	local sets = Bongos.SetBarSets(id, TLib.TCopy(parent.sets))
	sets.dock = parent.id
	sets.states = ''

	BActionBar.Create(id)
	parent:UpdateTabs()
end

function BActionBar:Undock()
	local id = self.id
	local parent = select(2, self:GetDock())

	self:Delete()
	BActionBar.Create(id)

	local ratio = UIParent:GetScale()
	local x, y = GetCursorPosition()

	self:ClearAllPoints()
	self:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / ratio, y / ratio)
	self:SavePosition()

	parent:UpdateTabs()
end


--[[ Tabs for docked bars ]]--

function BActionBar:UpdateTabs()
	local count = 0
	for i = 1, BActionBar.GetNumber() do
		local sets = Bongos.GetBarSets(i)
		if sets and sets.dock == self.id then
			count = count + 1
			if not self.tabs then self.tabs = {} end
	
			local tab = self.tabs[count]
			if not tab then
				tab = BTab.Get()
				self.tabs[count] = tab
			end
			tab:Set(self.dragFrame, i)
		end
	end
	
	if self.tabs then
		local numTabs = #self.tabs
		for i = count + 1, numTabs do
			self.tabs[i]:Release()
			self.tabs[i] = nil
		end
		self:LayoutTabs()
	end
end

function BActionBar:LayoutTabs()
	for i = 1, #self.tabs do
		local tab = self.tabs[i]
		tab:ClearAllPoints()
		tab:SetPoint('BOTTOMLEFT', self.dragFrame, 'TOPLEFT', tab:GetWidth() * (i-1), 0)
		tab:Show()
	end
end


--[[ Targeting ]]--

function BActionBar:SetUnit(buttonID, unit)
	self:SetAttribute('*unit' .. buttonID, unit)
end


--[[ Bar Numbers ]]--

function BActionBar.SetNumber(newSize)
	local oldSize = BActionBar.GetNumber()
	if oldSize ~= newSize then
		for i = 1, (oldSize or 10) do
			BActionBar.Get(i):Delete()
		end

		BActionDB.number = newSize
		for i = 1, (newSize or 10) do
			BActionBar.Create(i)
		end
	end
end

function BActionBar.GetNumber()
	return BActionDB.number or 10
end


--[[ Utility Functions ]]--

function BActionBar.ForAllShown(action, ...)
	for i = 1, BActionBar.GetNumber() do
		local bar = BActionBar.Get(i)
		if bar and bar:IsShown() then
			action(BActionBar.Get(i), ...)
		end
	end
end

function BActionBar.RefreshAll(showEmpty)
	for i = 1, BActionBar.GetNumber() do
		BActionBar.Get(i):Refresh(nil, showEmpty)
	end
end

function BActionBar.RefreshAllBindings()
	for i = 1, BActionBar.GetNumber() do
		BActionBar.Get(i):RefreshBindings()
	end
end


--[[ Startup ]]--

local function LoadDruidDefaults()
	Bongos.GetBarSets(1).states = 's0;s2;s4;s6;'

	Bongos.SetBarSets(7,  {dock = 1, vis = 1, states = 's3;'})
	Bongos.SetBarSets(8,  {dock = 1, vis = 1, states = 's7;'})
	Bongos.SetBarSets(9,  {dock = 1, vis = 1, states = 's1;'})
	Bongos.SetBarSets(10, {dock = 1, vis = 1, states = 's5;'})
end

local function LoadRogueDefaults()
	Bongos.GetBarSets(1).states = 's0;'

	Bongos.SetBarSets(7, {dock = 1, vis = 1, states = 's1;'})
	for i = 8, 10 do
		Bongos.SetBarSets(i, {dock = 1, vis = 1, states = ''})
	end
end

local function LoadWarriorDefaults()
	Bongos.GetBarSets(1).states = ''

	Bongos.SetBarSets(7,  {dock = 1, vis = 1, states = 's1;'})
	Bongos.SetBarSets(8,  {dock = 1, vis = 1, states = 's2;'})
	Bongos.SetBarSets(9,  {dock = 1, vis = 1, states = 's3;'})
	Bongos.SetBarSets(10, {dock = 1, vis = 1, states = ''})
end

local function LoadOtherClassDefaults()
	Bongos.GetBarSets(1).states = 's0;'

	for i = 7, 10 do
		Bongos.SetBarSets(i,  {dock = 1, vis = 1, states = ''})
	end
end

local function LoadSettings()
	ALL_STATES = ''
	local s, e = BState.GetStanceRange()
	for i = s, e do
		ALL_STATES = ALL_STATES .. 's' .. i .. ';'
	end

	local maxPage = BState.GetMaxPage() - 1
	for i = 1, maxPage do
		ALL_STATES = ALL_STATES .. 'p' .. i .. ';'
	end

	--load bar defaults
	if not Bongos.GetBarSets(1) then
		Bongos.SetBarSets(1, {
			[1] = "1;", [2] = "2;", [3] = "3;", [4] = "4;",  [5] = "5;",  [6] = "6;",
			[7] = "7;", [8] = "8;", [9] = "9;", [10] = "0;", [11] = "-;", [12] = "=;",
			x = 546, y = 37, vis = 1,
		})

		for i = 2, BState.GetMaxPage() do
			Bongos.SetBarSets(i, {dock = 1, vis = 1, states = format('p%d;', i-1)})
		end
		
		local class = select(2, UnitClass('player'))
		if class == 'DRUID' then
			LoadDruidDefaults()
		elseif class == 'WARRIOR' then
			LoadWarriorDefaults()
		elseif class == 'ROGUE' then
			LoadRogueDefaults()
		else
			LoadOtherClassDefaults()
		end
	end	
end

Bongos.AddStartup(function() 
	LoadSettings()
	for i = 1, BActionBar.GetNumber() do
		BActionBar.Create(i)
	end
end)