--[[
	State.lua
		Does state changing for class bars and paging
--]]

local CAT_STANCE = 3
local PROWL_STANCE = 7
local CLASS = select(2, UnitClass('player'))
local STANCES = BONGOS_STANCES[CLASS]
local MAX_PAGES = 6

local function Trans(state, map) 
	return format('%s:%s;', state, map) 
end

local function Range(s, e) 
	return format('%d-%d', s, e)
end


--[[ Stances ]]--

-- <oldStance><page>:<stance><page>;
local function GetStanceMap(stance)
	local stanceMap = ''
	local s, e = BState.GetStanceRange()

	for oStance = s, e do
		for page = 0, MAX_PAGES-1 do
			stanceMap = stanceMap .. Trans(oStance * 10 + page, stance * 10 + page)
		end
	end
	return stanceMap
end

-- <cat><page>:<prowl><page>;
local function GetProwlMaps()
	local prowlMap = ''
	local catMap = ''

	for page = 0, MAX_PAGES-1 do
		prowlMap = prowlMap .. Trans(CAT_STANCE * 10 + page, PROWL_STANCE * 10 + page)
		catMap = catMap .. Trans(PROWL_STANCE * 10 + page, CAT_STANCE * 10 + page)
	end
	return prowlMap, catMap
end

local function LoadStanceMaps(driver)
	local s, e = BState.GetStanceRange()

	if CLASS == 'DRUID' then
		for stance = s, e-1 do
			driver:SetAttribute('statemap-stance-' .. stance, GetStanceMap(stance))
		end

		local prowl, cat = GetProwlMaps()
		driver:SetAttribute('statemap-stealth-1', prowl)
		driver:SetAttribute('statemap-stealth-0', cat)
	elseif e > 0 then
		for stance = s, e do
			driver:SetAttribute('statemap-stance-' .. stance, GetStanceMap(stance))
		end
	end
end


--[[ Paging ]]--

-- <stance><minPage>-<stance><maxPage>:<stance><newPage>;
local function GetPageMap(page)
	local maxPage = BState.GetMaxPage()
	local pageMap = ''
	
	local s, e = BState.GetStanceRange()	
	for stance = s, e do
		pageMap = pageMap .. Trans(Range(stance*10, stance*10 + maxPage - 1), stance*10 + page-1)
	end
	return pageMap
end

local function LoadPageMaps(driver)
	for p = 1, MAX_PAGES do
		driver:SetAttribute('statemap-actionbar-' .. p, GetPageMap(p))
	end
end


--[[ Usable Functions ]]--

BState = {}

function BState.LoadMaps(driver)
	LoadStanceMaps(driver)
	LoadPageMaps(driver)

	driver:SetAttribute('state', BState.GetCurrentState())
end

function BState.GetCurrentState()
	local maxStance = select(2, BState.GetStanceRange())
	if maxStance > 0 then
		if CLASS == 'DRUID' and IsStealthed() and GetShapeshiftForm() == 3 and GetActionBarPage() == 1 then
			return PROWL_STANCE * 10
		else
			return GetShapeshiftForm() * 10 + GetActionBarPage() - 1
		end
	end
	return GetActionBarPage() - 1
end

function BState.GetStanceRange()
	if STANCES then
		if STANCES[0] then
			return 0, #STANCES
		end
		return 1, #STANCES
	end
	return 0, 0
end

function BState.GetMaxPage()
	return MAX_PAGES
end

--takes a statemap (s<number> or p<number>) and returns all the states that map to it
function BState.MapToStates(stateMap)
	local type, index = stateMap:match('(%w)(%d)')
	if type == 's' then
		return index * 10
	end
	
	local pages
	local s, e = BState.GetStanceRange()
	for stance = s, e do
		if pages then
			pages = pages .. ',' .. (stance*10 + index)
		else
			pages = (stance*10 + index)
		end
	end
	return pages
end

function BState.StateToMap(state)
	local remain = mod(state, 10)
	if remain == 0 then
		return 's' .. state/10
	end
	return 'p' .. remain
end

function BState.GetStateName(id)
	local type = id:sub(1, 1)
	local id = tonumber(id:sub(2))
	
	if type == 'p' then
		return format(BONGOS_PAGE, id + 1)
	end

	if STANCES then
		return STANCES[id] or BONGOS_UNKNOWN
	end
	return BONGOS_NORMAL
end