--[[
	BTab
--]]

BTab = CreateFrame('CheckButton')
local Tab_mt = {__index = BTab}


--[[ Rightclick Menu Functions ]]--

local function StateButton_OnEnter()
	GameTooltip:SetOwner(this, 'ANCHOR_TOPRIGHT')
	GameTooltip:SetText(this:GetText(), 1, 1, 1)

	if this:GetChecked() then
		GameTooltip:AddLine(BONGOS_STATE_HIDE)
		GameTooltip:AddLine(BONGOS_STATE_INVERSE_SELECT)
	else
		GameTooltip:AddLine(BONGOS_STATE_SHOW)
		GameTooltip:AddLine(BONGOS_STATE_SELECT_ONLY)
	end
	GameTooltip:Show()
end

local function StateButton_OnLeave()
	GameTooltip:Hide()
end

local function StateButton_OnClick()
	local menu = this:GetParent()
	local name = menu:GetName()
	local enable = this:GetChecked()

	if IsControlKeyDown() then
		local s,e = BState.GetStanceRange()
		for i = s, e do
			local button = getglobal(name .. 'Stance' .. i)
			if button ~= this then
				menu.frame:AddState('s' .. i, not enable)
				button:SetChecked(not enable)
			end
		end

		local maxPage = BState.GetMaxPage()
		for i = 2, maxPage do
			local button = getglobal(name .. 'Page' .. i)
			if button ~= this then
				menu.frame:AddState('p' .. i-1, not enable)
				button:SetChecked(not enable)
			end
		end
	end
	menu.frame:AddState(this.id, enable)
	StateButton_OnEnter()
end

local function StateButton_Create(name, id, parent)
	local button = CreateFrame('CheckButton', name, parent, 'GooeyCheckButton')
	button.id = id
	button:SetText(BState.GetStateName(id))
	button:SetScript('OnClick', StateButton_OnClick)
	button:SetScript('OnEnter', StateButton_OnEnter)
	button:SetScript('OnLeave', StateButton_OnLeave)
	
	return button
end

local function CreateConfigMenu(name)
	local menu = CreateFrame("Button", name, UIParent, "BongosRightClickMenu")

	--stances
	local s, e = BState.GetStanceRange()
	for i = s, e do
		local button = StateButton_Create(name .. 'Stance' .. i, 's' .. i, menu)
		if i == s then
			button:SetPoint('TOPLEFT', menu, 'TOPLEFT', 6, -28)
		else
			button:SetPoint('TOP', name .. 'Stance' .. (i-1), 'BOTTOM', 0, 2)
		end
	end

	--pages
	local maxPage = BState.GetMaxPage()
	for i = 2, maxPage do
		local button = StateButton_Create(name .. 'Page' .. i, 'p' .. i-1, menu)
		if i > 2 then
			button:SetPoint('TOP', name .. 'Page' .. (i-1), 'BOTTOM', 0, 2)
		else
			button:SetPoint('TOP', name .. 'Stance' .. e, 'BOTTOM', 0, 2)
		end
	end

	menu:SetWidth(180)
	menu:SetHeight(32 + (e - s + maxPage) * 30)
	
	return menu
end


--[[ Constructor ]]--

local function OnClick() this:OnClick(arg1) this:OnEnter() end
local function OnEnter() this:OnEnter() end
local function OnLeave() this:OnLeave() end

local function OnDragStart()
	this:OnLeave()
	this:StartMoving()
	this.moving = true
end

local function OnDragStop()
	this:OnDragStop()
	GameTooltip:Hide()
	this.moving = nil
end

local function Tab_Create(lastCreated)
	local tab = CreateFrame('Button', nil, nil, 'UIPanelButtonTemplate')
	setmetatable(tab, Tab_mt)

	tab:SetWidth(24)
	tab:SetHeight(24)
	tab:SetFrameStrata('DIALOG')
	tab:SetMovable(true)
	tab:SetToplevel(true)
	tab:SetAlpha(0.7)

	tab:RegisterForDrag("LeftButton")
	tab:RegisterForClicks("anyUp")

	tab:SetScript('OnClick', OnClick)
	tab:SetScript('OnDragStart', OnDragStart)
	tab:SetScript('OnDragStop', OnDragStop)
	tab:SetScript('OnEnter', OnEnter)
	tab:SetScript('OnLeave', OnLeave)

	return tab
end


--[[ Methods ]]--

function BTab.Get()
	return TPool.Get('BTab', Tab_Create)	
end

function BTab:Set(parent, id)
	self:SetParent(parent)
	self:SetID(id)
	self:Update()
end

function BTab:Release()
	return TPool.Release(self, 'BTab')
end

function BTab:Update()
	self:SetText(self:GetID())
end

--Called when the right click menu is shown, loads the correct values to the checkbuttons/sliders/text
function BTab:ShowMenu()
	local name = 'BongosActionBarTabMenu'
	local menu = getglobal(name) or CreateConfigMenu(name)

	menu:SetText(format('Action Bar %s', self:GetID()))
	menu.frame = BActionBar.Get(self:GetID())
	menu.onShow = 1

	local s, e = BState.GetStanceRange()
	for i = s, e do
		getglobal(name .. 'Stance' .. i):SetChecked(menu.frame:HasState('s' .. i))
	end

	local maxPage = BState.GetMaxPage()
	for i = 2, maxPage do
		getglobal(name .. 'Page' .. i):SetChecked(menu.frame:HasState('p' .. i-1))
	end

	self:DisplayMenu(menu)
	
	if BongosActionBarMenu then
		BongosActionBarMenu:Hide()
	end

	menu.onShow = nil
end

function BTab:DisplayMenu(menu)
	local dragFrame = self:GetParent()
	local ratio = UIParent:GetScale() / dragFrame:GetEffectiveScale()
	local x = dragFrame:GetLeft()
	local y = dragFrame:GetTop()

	menu:ClearAllPoints()
	menu:SetPoint("TOPRIGHT", UIParent, "BOTTOMLEFT", x  / ratio, y / ratio)
	menu:Show()
end


--[[ OnX ]]--

function BTab:OnClick(mouseButton)
	if arg1 == 'RightButton' then
		self:ShowMenu()
	end
end

function BTab:OnEnter()
	GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	GameTooltip:SetText('action bar ' .. self:GetID(), 1, 1, 1)
	GameTooltip:AddLine(BONGOS_SHOW_CONFIG)
	GameTooltip:AddLine(BONGOS_DOCK_HELP)
	GameTooltip:AddLine(BONGOS_UNDOCK_HELP)

	local bar = BActionBar.Get(self:GetID())
	local used
	local s,e = BState.GetStanceRange()
	for i = s, e do
		local stance = 's' .. i
		if bar:HasState(stance) then
			used = true
			GameTooltip:AddLine(format(BONGOS_STATE_SHOWN_IN, BState.GetStateName(stance)), 0, 1, 0)
		end
	end

	local maxPage = BState.GetMaxPage()
	for i = 2, maxPage do
		local page = 'p' .. i-1
		if bar:HasState(page) then
			used = true
			GameTooltip:AddLine(format(BONGOS_STATE_SHOWN_ON_PAGE, i), 0, 1, 0)
		end
	end
	if not used then
		GameTooltip:AddLine(BONGOS_STATE_UNUSED, 1, 0.2, 0.2)
	end
	GameTooltip:Show()
end

function BTab:OnLeave()
	GameTooltip:Hide()
end

function BTab:OnDragStop()
	self:StopMovingOrSizing()
		
	local bar = BActionBar.Get(self:GetID())
	if not MouseIsOver(self:GetParent()) or MouseIsOver(bar.sets.dock) then
		BActionBar.Get(self:GetID()):Undock()
	else
		self:GetParent().parent:UpdateTabs()
	end
	
	for i = 1, BActionBar.GetNumber() do
		local otherBar = BActionBar.Get(i)
		if bar ~= otherBar and MouseIsOver(otherBar) then
			bar:Dock(otherBar)
			otherBar:UpdateTabs()
			return
		end
	end
end